import spatiafi.gdal_auth
from spatiafi.auth import authenticate
from spatiafi.session import get_session

try:
    # -- Distribution mode --
    # import from _version.py generated by setuptools_scm during release
    from ._version import version as __version__
except ImportError:
    # -- Source mode --
    # use setuptools_scm to get the current version from src using git
    from setuptools_scm import get_version as _gv

    __version__ = _gv(relative_to=__file__, search_parent_directories=True)
